//
// The editor instance this plugin is used with
// needs to have an array with objects of type
// {'description' : '', 'code' : ''}
// at *editor.config.pluginOptions.epplaceholder.placeholder*.
//
// E.g.: editor.config.pluginOptions.epplaceholder.placeholder = [{'description' : 'Desc0', 'code' : '#Test1'}, {'description' : 'Desc1', 'code' : '#Test2'}];
//

/*
 * @dictionary		ep.dict
 *
 * @translation		{TLEs}
 */
(function($){
	// Name of our plugin.
	var pluginName = 'epplaceholder',
		epDict = ep.dict('ep.dict'),
		translations = {
			tPlaceholder: epDict.translate('{TLEs}')
		};

	// Add button to UI.
	var addPlaceholderRichCombo = function (editor, placeholder) {
		editor.ui.addRichCombo(pluginName, {
				label: translations.tPlaceholder,
				title : translations.tPlaceholder,
				panel :{
					css : editor.skin.editor.css.concat(editor.config.contentsCss),
					multiSelect : false
				},
				className : 'cke_format cke_placeholder', // Inherit from *format* plugin. (see presets.css)
				placeholder : placeholder, // Pass placeholders to richcombo instance. (Used in *init()*.)
				init : function () {
					// Build *placeholder* group.
					var i;
					for (i = 0; i < this.placeholder.length; i++) {
						var placeholder = this.placeholder[i];
						// Add the placeholder entry to the panel list.
						this.add(placeholder.code, '<p>' + placeholder.description + '</p>', placeholder.description);
					}

					// Overwrite *onClick* function of *this._.list*
					// such that clicked items do no stay *marked*.
					// Another possibilty would be to use a *timeout* in *onClick*
					// and do *this.setValue('');* there. (Dirty too...)
					this._.list.onClick = $.proxy(function (value, marked) {
						// Standard.
 						var me = this;
 						me.document.getWindow().focus();

 						// Added by my lovely self.
						me.unmarkAll();
						marked = false;

						// Standard.
 						me.onClick && me.onClick.call( me, value, marked );
						me._.panel.hide( false );
 					}, this);
				},
				onClick : function (value, marked) {
					// Insert *placeholder* code.
					editor.focus();
					editor.fire('saveSnapshot');
					editor.insertText(value);
					editor.fire('saveSnapshot');
				}
			});
	};

	// Add plugin to CKEDITOR.
	CKEDITOR.plugins.add(pluginName, {
		requires : ['richcombo'],
		init : function (editor) {
			var placeholder = editor.config.pluginOptions && editor.config.pluginOptions.epplaceholder && editor.config.pluginOptions[pluginName].placeholder;
			// No placeholders, no insert placeholder button...
			placeholder && addPlaceholderRichCombo(editor, placeholder);
		}
	});

})(jQuery);