/*global define,window,main*/

define([
    'testsuite/qunit',
    'jquery',
    'de_epages/mediagallery/filemanager/main-view',

    '$ready!'
], function (QUnit, $, MainView) {
    'use strict';

    QUnit.module('de_epages/mediagallery/filemanager/main-view', {
        setup: function () {
            // fake data for ajax requests
            var filemanager = $('<div class="filemanager" />').appendTo('#qunit-fixture'),
                initArray = [{
                    "currentGallerySize": "1344",
                    "children": [{
                        "files": [{
                            "width": null,
                            "extension": ".swf",
                            "name": "promo.swf",
                            "path": "/promo.swf",
                            "height": null,
                            "directory": "/MediaGallery",
                            "modified": 1200994146,
                            "size": 67089,
                            "fullpath": "/WebRoot/Store/Shops/DemoShop/MediaGallery/promo.swf",
                            "selectable": 0,
                            "mimetype": "application/x-shockwave-flash"
                        }, {
                            "width": "240",
                            "extension": ".gif",
                            "name": "StartPagePromotion.gif",
                            "path": "/StartPagePromotion.gif",
                            "height": "113",
                            "directory": "/MediaGallery",
                            "modified": 1200994146,
                            "size": 14453,
                            "preview": "/WebRoot/Store/Shops/DemoShop/MediaGallery/StartPagePromotion.gif",
                            "fullpath": "/WebRoot/Store/Shops/DemoShop/MediaGallery/StartPagePromotion.gif",
                            "selectable": 1,
                            "mimetype": "image/gif"
                        }],
                        "children": [{
                            "files": [],
                            "children": [],
                            "metadata": {
                                "displayname": "neu",
                                "type": "MediaGallery",
                                "path": "/neu"
                            },
                            "data": "neu",
                            "state": null
                        }],
                        "metadata": {
                            "displayname": "MediaGallery",
                            "type": "MediaGallery",
                            "path": ""
                        },
                        "data": "MediaGallery",
                        "state": "closed"
                    }, {
                        "files": [],
                        "children": [{
                            "files": [],
                            "children": [],
                            "metadata": {
                                "displayname": "Kunst, Kultur und Bildung",
                                "type": "Header",
                                "path": "/StyleGroups/Branches/StyleGroups/ArtAndEducation"
                            },
                            "data": "ArtAndEducation",
                            "state": null
                        }, {
                            "files": [],
                            "children": [],
                            "metadata": {
                                "displayname": "Auto und Transport",
                                "type": "Header",
                                "path": "/StyleGroups/Branches/StyleGroups/Automotive"
                            },
                            "data": "Automotive",
                            "state": null
                        }],
                        "metadata": {
                            "displayname": "Kopfzeilenbilder",
                            "type": "Header",
                            "path": "/StyleGroups/Branches/StyleGroups"
                        },
                        "data": "HeaderImages",
                        "state": "closed"
                    }, {
                        "files": [],
                        "children": [{
                            "files": [],
                            "children": [],
                            "metadata": {
                                "displayname": "Kunst, Kultur und Bildung",
                                "type": "Background",
                                "path": "/StyleGroups/Branches/StyleGroups/ArtAndEducation"
                            },
                            "data": "ArtAndEducation",
                            "state": null
                        }],
                        "metadata": {
                            "displayname": "Hintergrundbilder",
                            "type": "Background",
                            "path": "/StyleGroups/Branches/StyleGroups"
                        },
                        "data": "BackgroundImages",
                        "state": "closed"
                    }, {
                        "files": [],
                        "children": [{
                            "files": [],
                            "children": [],
                            "metadata": {
                                "displayname": "Gebäude",
                                "type": "ImageDB",
                                "path": "/ImageDBGroups/Buildings"
                            },
                            "data": "Buildings",
                            "state": null
                        }],
                        "metadata": {
                            "displayname": "{ImageDB}",
                            "type": "ImageDB",
                            "path": "ImageDBGroups"
                        },
                        "data": "ImageDB",
                        "state": "closed"
                    }],
                    "maxGallerySize": "102400"
                }],
                directoryListArray = [{
                    "files": [],
                    "children": [],
                    "metadata": {
                        "displayname": "Kunst, Kultur und Bildung",
                        "type": "Background",
                        "path": "/StyleGroups/Branches/StyleGroups/ArtAndEducation"
                    },
                    "data": "ArtAndEducation",
                    "state": null
                }],
                createDirectoryArray = [{
                    "currentGallerySize": "3143",
                    "children": [{
                        "files": [],
                        "children": [],
                        "metadata": {
                            "type": "MediaGallery",
                            "path": "/Neuen_Ordner_anlegen"
                        },
                        "data": "Neuen_Ordner_anlegen",
                        "state": null
                    }]
                }],
                renameDirectoryObj = {
                    "ResultingPath": "/newName",
                    "ResultingDirectory": "newName"
                },
                renameFileObj = {
                    "ResultingPath": "//StartPagePromotion.gif",
                    "ResultingFilename": "StartPagePromotion.gif",
                    "ResultingFullPath": "/WebRoot/Store/Shops/DemoShop/MediaGallery///StartPagePromotion.gif"
                },
                deleteFolderObj = {
                    "currentGallerySize": "3165"
                },
                deleteFileObj = {
                    "currentGallerySize": "1180"
                };

            // fake ep.ajax for the different requests
            $.ajax = function (o) {
                var result = $.Deferred();

                if (o.data.ViewAction === "JSONInitMediaGallery") {
                    result.resolve(initArray);
                }

                if (o.data.ViewAction === "JSONGetDirectoryList") {
                    result.resolve(directoryListArray);
                }

                switch (o.data.ChangeAction) {
                case "JSONCreateDirectory":
                    result.resolve(createDirectoryArray);
                    break;
                case "JSONRenameDirectory":
                    result.resolve(renameDirectoryObj);
                    break;
                case "JSONRenameFile":
                    result.resolve(renameFileObj);
                    break;
                case "JSONDeleteDirectory":
                    result.resolve(deleteFolderObj);
                    break;
                case "JSONDeleteFile":
                    result.resolve(deleteFileObj);
                    break;
                }

                return result;
            };

            window.main = new MainView({
                element: $('.filemanager'),
                currentFolder: 'MediaGallery',
                maxRender: 5
            });

            QUnit.strictEqual(
                $('.ep-mediagallery-toolbar').length,
                1,
                'Create filemanager and append it to given element'
            );

        },
        teardown: function () {
            $('.ep-mediagallery').empty();

            QUnit.strictEqual(
                $('.ep-mediagallery-toolbar').length,
                0,
                'Remove filemanager'
            );
        }
    });

    QUnit.test('view methods', 16 + 2, function () {
        $.each([
            'initialize',
            'render',
            'changePreview',
            'openFolder',
            'folderEditable',
            'updateListView',
            'removeExistingFile',
            'updateUsage',
            'addTreeNode',
            'renameTreeNode',
            'removeTreeNode',
            'deleteItems',
            'createNewFolder',
            'getCurrentUsage',
            'getAllSelectedElements',
            'getSelectedElements'
        ], function (i, method) {
            QUnit.strictEqual(
                $.isFunction(main[method]),
                true,
                method + ' is a method of main view'
            );
        });
    });

    QUnit.test('view functionality', 9 + 2, function () {
        QUnit.strictEqual(
            $('.ep-mediagallery-preview img').length,
            0,
            'Preview is empty'
        );

        $('.ep-mediagallery-folderContainer').trigger('click');

        QUnit.strictEqual(
            $('.ep-mediagallery-preview img').length,
            1,
            'Show first folder in preview when it is selected'
        );

        main.deleteItems();
        $('.ui-dialog-buttonset button:first').trigger('click');

        QUnit.strictEqual(
            $('.ep-mediagallery-itemList li').length,
            2,
            'Remove selected file'
        );

        QUnit.strictEqual(
            main.getCurrentUsage(),
            '3165',
            'Get current usage'
        );

        $('.ep-mediagallery-fileContainer:first').trigger('click');

        QUnit.strictEqual(
            $('.ep-mediagallery-fileContainer.ui-selected').length,
            1,
            'Select first file'
        );

        QUnit.strictEqual(
            main.getSelectedElements().length,
            0,
            'Get selected elements (only that are selectable)'
        );

        QUnit.strictEqual(
            main.getAllSelectedElements().length,
            1,
            'Get selected elements'
        );

        $('.ep-mediagallery-newFolderButton').trigger('click');

        QUnit.strictEqual(
            $('.ep-mediagallery-itemList li').length,
            3,
            'Create new folder'
        );

        $('.ep-mediagallery-treeHead:last').trigger('click');

        QUnit.strictEqual(
            $('.ep-mediagallery-listView h2').text(),
            'Bilddatenbank',
            'Open folder Bilddatenbank'
        );

    });

});